package main

import (
    "fmt"
    "image/color"
    "log"
    "os"
    "os/user"
    "path/filepath"
    "sort"
    "time"

    "gioui.org/app"
    "gioui.org/layout"
    "gioui.org/op"
    "gioui.org/unit"
    "gioui.org/widget"
    "gioui.org/widget/material"

    bolt "github.com/etcd-io/bbolt"
    "git.wow.st/gmp/logbook"
)

var (
    btnPrevWorkouts    widget.Clickable
    btnPredictStrength widget.Clickable
    btnMeasurements    widget.Clickable
    btnPlanWorkout     widget.Clickable
    btnLeft            widget.Clickable
    btnRight           widget.Clickable
    list               = new(layout.List) // pointer to layout.List
)

type page int

const (
    pageLanding page = iota
    pagePlanWorkout
)

func main() {
    go func() {
        window := new(app.Window)
        if err := run(window); err != nil {
            log.Fatal(err)
        }
        os.Exit(0)
    }()
    app.Main()
}

func openStandardDB() (*bolt.DB, error) {
    usr, err := user.Current()
    if err != nil {
        return nil, err
    }
    dir := filepath.Join(usr.HomeDir, ".config", "logbook")
    if err := os.MkdirAll(dir, 0700); err != nil {
        return nil, err
    }
    dbPath := filepath.Join(dir, "logbook.db")
    db, err := bolt.Open(dbPath, 0600, nil)
    if err != nil {
        return nil, err
    }
    return db, nil
}

func run(window *app.Window) error {
    theme := material.NewTheme()
    var ops op.Ops

    db, err := openStandardDB()
    if err != nil {
        return fmt.Errorf("failed to open DB: %w", err)
    }
    defer db.Close()
    workoutRepo, err := logbook.NewWorkoutRepository(db)
    if err != nil {
        return fmt.Errorf("failed to create workout repo: %w", err)
    }
    setRepo, err := logbook.NewSetRepository(db)
    if err != nil {
        return fmt.Errorf("failed to create set repo: %w", err)
    }

    // Navigation state
    currentPage := pageLanding
    var planPage *planWorkoutPage

    // For landing page state
    var planned []*logbook.Workout
    var workoutIdx int

    // Preload planned workouts for landing page
    reloadPlanned := func() {
        now := time.Now()
        allWorkouts, err := workoutRepo.FindAll()
        if err != nil {
            planned = nil
            workoutIdx = 0
            return
        }
        planned = filterPlannedWorkoutsAfter(allWorkouts, now)
        sort.Slice(planned, func(i, j int) bool {
            return planned[i].Date.Before(planned[j].Date)
        })
        workoutIdx = 0
    }
    reloadPlanned()

    for {
        e := window.Event()
        switch e := e.(type) {
        case app.DestroyEvent:
            return e.Err
        case app.FrameEvent:
            gtx := app.NewContext(&ops, e)

            switch currentPage {
            case pageLanding:
                // Click handling flags
                planWorkoutClicked := false

                layout.Flex{
                    Axis: layout.Vertical,
                }.Layout(gtx,
                    // Top navigation bar
                    layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                        return layout.Flex{Axis: layout.Horizontal}.Layout(gtx,
                            layout.Rigid(material.Button(theme, &btnPrevWorkouts, "Previous Workouts").Layout),
                            layout.Rigid(material.Button(theme, &btnPredictStrength, "Predict Strength").Layout),
                            layout.Rigid(material.Button(theme, &btnMeasurements, "Measurements").Layout),
                            layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                                dims := material.Button(theme, &btnPlanWorkout, "Plan Workout").Layout(gtx)
                                if btnPlanWorkout.Clicked(gtx) {
                                    planWorkoutClicked = true
                                }
                                return dims
                            }),
                        )
                    }),
                    // Main content: Next planned workout
                    layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                        if len(planned) == 0 {
                            return material.H5(theme, "No planned workouts.").Layout(gtx)
                        }
                        w := planned[workoutIdx]
                        return layout.Flex{Axis: layout.Vertical}.Layout(gtx,
                            layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                                return material.H4(theme, "Next Workout: "+w.Date.Format("Mon Jan 2, 2006 3:04 PM")).Layout(gtx)
                            }),
                            layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                                return material.H5(theme, "Type: "+w.Type).Layout(gtx)
                            }),
                            // Planned sets, one per line
                            layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                                sets := getPlannedSets(setRepo, w.SetIDs)
                                return list.Layout(gtx, len(sets), func(gtx layout.Context, i int) layout.Dimensions {
                                    set := sets[i]
                                    return material.Body1(theme, formatSet(set)).Layout(gtx)
                                })
                            }),
                        )
                    }),
                    // Bottom navigation: left/right arrows
                    layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                        leftClicked := false
                        rightClicked := false
                        dims := layout.Flex{Axis: layout.Horizontal, Spacing: layout.SpaceBetween}.Layout(gtx,
                            layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                                btn := material.Button(theme, &btnLeft, "←")
                                if workoutIdx == 0 {
                                    btn.Background = color.NRGBA{A: 80}
                                }
                                d := btn.Layout(gtx)
                                if btnLeft.Clicked(gtx) {
                                    leftClicked = true
                                }
                                return d
                            }),
                            layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                                btn := material.Button(theme, &btnRight, "→")
                                if workoutIdx == len(planned)-1 {
                                    btn.Background = color.NRGBA{A: 80}
                                }
                                d := btn.Layout(gtx)
                                if btnRight.Clicked(gtx) {
                                    rightClicked = true
                                }
                                return d
                            }),
                        )
                        // Handle navigation after layout
                        if leftClicked && workoutIdx > 0 {
                            workoutIdx--
                        }
                        if rightClicked && workoutIdx < len(planned)-1 {
                            workoutIdx++
                        }
                        return dims
                    }),
                )

                // Handle navigation to Plan Workout page after layout
                if planWorkoutClicked {
                    planPage = newPlanWorkoutPage()
                    currentPage = pagePlanWorkout
                }

            case pagePlanWorkout:
                planPage.Layout(gtx, theme, setRepo, workoutRepo, func() {
                    reloadPlanned()
                    currentPage = pageLanding
                })
            }

            e.Frame(gtx.Ops)
        }
    }
}

func filterPlannedWorkoutsAfter(workouts []*logbook.Workout, after time.Time) []*logbook.Workout {
    var planned []*logbook.Workout
    for _, w := range workouts {
        if w.Date.After(after) {
            planned = append(planned, w)
        }
    }
    return planned
}

// Use setRepo.GetByID for efficient lookup
func getPlannedSets(setRepo *logbook.SetRepository, setIDs []string) []logbook.Set {
    var sets []logbook.Set
    for _, id := range setIDs {
        set, err := setRepo.GetByID(id)
        if err == nil && set != nil {
            sets = append(sets, *set)
        }
    }
    return sets
}

func formatSet(set logbook.Set) string {
    if set.Weight > 0 {
        return fmt.Sprintf("%s: %d reps @ %.0f lb", set.Exercise, set.Reps, set.Weight)
    }
    return fmt.Sprintf("%s: %d reps (bodyweight)", set.Exercise, set.Reps)
}

// --- Plan Workout Page Implementation ---

type planWorkoutPage struct {
    dateEditor     widget.Editor
    typeEditor     widget.Editor
    addSetBtn      widget.Clickable
    saveBtn        widget.Clickable
    sets           []plannedSet
    setEditors     []plannedSetEditors
    errorMsg       string
    successMsg     string
    backBtn        widget.Clickable
}

type plannedSet struct {
    Exercise string
    Reps     int
    Weight   float64
}

type plannedSetEditors struct {
    exerciseEditor widget.Editor
    repsEditor     widget.Editor
    weightEditor   widget.Editor
}

func newPlanWorkoutPage() *planWorkoutPage {
    p := &planWorkoutPage{}
    p.dateEditor.SingleLine = true
    p.dateEditor.SetText(time.Now().Format("2006-01-02"))
    p.typeEditor.SingleLine = true
    p.setEditors = []plannedSetEditors{newPlannedSetEditors()}
    return p
}

func newPlannedSetEditors() plannedSetEditors {
    var e plannedSetEditors
    e.exerciseEditor.SingleLine = true
    e.repsEditor.SingleLine = true
    e.weightEditor.SingleLine = true
    return e
}

func (p *planWorkoutPage) Layout(gtx layout.Context, th *material.Theme, setRepo *logbook.SetRepository, workoutRepo *logbook.WorkoutRepository, navBack func()) layout.Dimensions {
    var ls = new(layout.List)
    ls.Axis = layout.Vertical
    // Handle Add Set button
    if p.addSetBtn.Clicked(gtx) {
        p.setEditors = append(p.setEditors, newPlannedSetEditors())
    }

    // Handle Save button
    if p.saveBtn.Clicked(gtx) {
        p.errorMsg = ""
        p.successMsg = ""
        // Parse date
        date, err := time.Parse("2006-01-02", p.dateEditor.Text())
        if err != nil {
            p.errorMsg = "Invalid date format (use YYYY-MM-DD)"
        } else {
            // Collect sets
            var setIDs []string
            for _, ed := range p.setEditors {
                exercise := ed.exerciseEditor.Text()
                reps, _ := parseInt(ed.repsEditor.Text())
                weight, _ := parseFloat(ed.weightEditor.Text())
                if exercise == "" || reps == 0 {
                    continue // skip incomplete sets
                }
                set := &logbook.Set{
                    Exercise: exercise,
                    Reps:     reps,
                    Weight:   weight,
                    Date:     date,
                    Plan:     true,
                }
                if err := setRepo.Save(set); err == nil {
                    setIDs = append(setIDs, set.ID)
                }
            }
            if len(setIDs) == 0 {
                p.errorMsg = "No valid sets entered."
            } else {
                workout := &logbook.Workout{
                    Date:   date,
                    Type:   p.typeEditor.Text(),
                    SetIDs: setIDs,
                }
                if err := workoutRepo.Save(workout); err != nil {
                    p.errorMsg = "Failed to save workout: " + err.Error()
                } else {
                    p.successMsg = "Workout planned!"
                    navBack()
                }
            }
        }
    }

    // Handle Back button
    if p.backBtn.Clicked(gtx) {
        navBack()
    }

    return layout.Flex{Axis: layout.Vertical}.Layout(gtx,
        layout.Rigid(material.H4(th, "Plan Workout").Layout),
        layout.Rigid(material.Body1(th, "Date (YYYY-MM-DD):").Layout),
        layout.Rigid(material.Editor(th, &p.dateEditor, "2025-05-05").Layout),
        layout.Rigid(material.Body1(th, "Type:").Layout),
        layout.Rigid(material.Editor(th, &p.typeEditor, "e.g. Strength, Hypertrophy").Layout),
        layout.Rigid(material.Body1(th, "Sets:").Layout),
        layout.Rigid(func(gtx layout.Context) layout.Dimensions {
            return ls.Layout(gtx, len(p.setEditors), func(gtx layout.Context, i int) layout.Dimensions {
                ed := &p.setEditors[i]
                return layout.Flex{Axis: layout.Horizontal}.Layout(gtx,
                    layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                        return layout.Inset{Right: unit.Dp(8)}.Layout(gtx, material.Editor(th, &ed.exerciseEditor, "Exercise").Layout)
                    }),
                    layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                        return layout.Inset{Right: unit.Dp(8)}.Layout(gtx, material.Editor(th, &ed.repsEditor, "Reps").Layout)
                    }),
                    layout.Rigid(func(gtx layout.Context) layout.Dimensions {
                        return material.Editor(th, &ed.weightEditor, "Weight (lb)").Layout(gtx)
                    }),
                )
            })
        }),
        layout.Rigid(material.Button(th, &p.addSetBtn, "Add Set").Layout),
        layout.Rigid(material.Button(th, &p.saveBtn, "Save Workout").Layout),
        layout.Rigid(func(gtx layout.Context) layout.Dimensions {
            if p.errorMsg != "" {
                return material.Body1(th, "[Error] "+p.errorMsg).Layout(gtx)
            }
            if p.successMsg != "" {
                return material.Body1(th, p.successMsg).Layout(gtx)
            }
            return layout.Dimensions{}
        }),
        layout.Rigid(material.Button(th, &p.backBtn, "Back").Layout),
    )
}

func parseInt(s string) (int, error) {
    var i int
    _, err := fmt.Sscanf(s, "%d", &i)
    return i, err
}
func parseFloat(s string) (float64, error) {
    var f float64
    _, err := fmt.Sscanf(s, "%f", &f)
    return f, err
}

